@@ -31,10 +31,10 @@ const BIAS: u8 = 65;
 // The rope will become less efficient after the string is 2 ^ ROPE_MAX_HEIGHT nodes.
 
 #[cfg(debug_assertions)]
-const NODE_STR_SIZE: usize = 10;
+pub(crate) const NODE_STR_SIZE: usize = 10;
 #[cfg(not(debug_assertions))]
-const NODE_STR_SIZE: usize = 392;
-// const NODE_STR_SIZE: usize = XX_SIZE;
+pub(crate) const NODE_STR_SIZE: usize = 392;
+// pub(crate) const NODE_STR_SIZE: usize = XX_SIZE;
 
 const MAX_HEIGHT: usize = 20;//NODE_STR_SIZE / mem::size_of::<SkipEntry>();
 const MAX_HEIGHT_U8: u8 = MAX_HEIGHT as u8;
@@ -84,7 +84,7 @@ pub(super) struct Node {
     pub(super) str: GapBuffer<NODE_STR_SIZE>,
 
     // Height of nexts array.
-    height: u8,
+    pub(super) height: u8,
 
     // #[repr(align(std::align_of::<SkipEntry>()))]
 
@@ -318,7 +318,7 @@ impl JumpRope {
     // Internal function for navigating to a particular character offset in the rope.  The function
     // returns the list of nodes which point past the position, as well as offsets of how far into
     // their character lists the specified characters are.
-    pub(crate) fn cursor_at_char(&self, char_pos: usize) -> RopeCursor {
+    pub(crate) fn cursor_at_char(&self, char_pos: usize, stick_end: bool) -> RopeCursor {
         assert!(char_pos <= self.len_chars());
 
         let mut e: *const Node = &self.head;
@@ -332,7 +332,7 @@ impl JumpRope {
             let en = unsafe { &*e };
             let next = en.nexts()[height];
             let skip = next.skip_chars;
-            if offset > skip {
+            if offset > skip || (!stick_end && offset == skip && !next.node.is_null()) {
                 // Go right.
                 assert!(e == &self.head || !en.str.is_empty());
                 offset -= skip;
@@ -361,7 +361,7 @@ impl JumpRope {
     }
 
     fn cursor_at_end(&self) -> RopeCursor {
-        self.cursor_at_char(self.len_chars())
+        self.cursor_at_char(self.len_chars(), true)
     }
 
     // Internal fn to create a new node at the specified iterator filled with the specified
@@ -588,7 +588,10 @@ impl JumpRope {
                     s.skip_chars -= removed;
                 }
             } else {
-                // Remove the node from the skip list.
+                // Remove the node from the skip list. This works because the cursor must be
+                // pointing from the previous element to the start of this element.
+                assert_ne!(cursor.0[0].node, node);
+
                 for i in 0..(*node).height as usize {
                     let s = &mut (*cursor.0[i].node).nexts_mut()[i];
                     s.node = (*node).nexts_mut()[i].node;
@@ -611,7 +614,7 @@ impl JumpRope {
     }
 
     fn eq_str(&self, mut other: &str) -> bool {
-        if self.len() != other.len() { return false; }
+        if self.len_bytes() != other.len() { return false; }
 
         for s in self.chunks().strings() {
             let (start, rem) = other.split_at(s.len());
@@ -718,6 +721,7 @@ impl Display for JumpRope {
     }
 }
 
+// I don't know why I need all three of these, but I do.
 impl PartialEq<str> for JumpRope {
     fn eq(&self, other: &str) -> bool {
         self.eq_str(other)
@@ -728,6 +732,11 @@ impl PartialEq<&str> for JumpRope {
         self.eq_str(*other)
     }
 }
+impl PartialEq<String> for JumpRope {
+    fn eq(&self, other: &String) -> bool {
+        self.eq_str(other.as_str())
+    }
+}
 
 impl<'a> Extend<&'a str> for JumpRope {
     fn extend<T: IntoIterator<Item = &'a str>>(&mut self, iter: T) {
@@ -772,7 +781,7 @@ impl JumpRope {
         if contents.is_empty() { return; }
         pos = std::cmp::min(pos, self.len_chars());
 
-        let mut cursor = self.cursor_at_char(pos);
+        let mut cursor = self.cursor_at_char(pos, true);
         unsafe { self.insert_at_cursor(&mut cursor, contents); }
 
         debug_assert_eq!(cursor.global_char_pos(self.head.height), pos + count_chars(contents));
@@ -796,7 +805,8 @@ impl JumpRope {
         range.end = range.end.min(self.len_chars());
         if range.start >= range.end { return; }
 
-        let mut cursor = self.cursor_at_char(range.start);
+        // We need to stick_end so we can delete entries.
+        let mut cursor = self.cursor_at_char(range.start, true);
         unsafe { self.del_at_cursor(&mut cursor, range.end - range.start); }
 
         debug_assert_eq!(cursor.global_char_pos(self.head.height), range.start);
@@ -818,7 +828,7 @@ impl JumpRope {
         let pos = usize::min(range.start, len);
         let del_len = usize::min(range.end, len) - pos;
 
-        let mut cursor = self.cursor_at_char(pos);
+        let mut cursor = self.cursor_at_char(pos, true);
         if del_len > 0 {
             unsafe { self.del_at_cursor(&mut cursor, del_len); }
         }
@@ -844,65 +854,87 @@ impl JumpRope {
     /// assert_eq!(str.len(), 11); // 11 bytes over the wire
     ///
     /// let rope = JumpRope::from(str);
-    /// assert_eq!(rope.len(), str.len());
+    /// assert_eq!(rope.len_bytes(), str.len());
     /// ```
-    pub fn len(&self) -> usize { self.num_bytes }
+    pub fn len_bytes(&self) -> usize { self.num_bytes }
 
     /// Returns `true` if the rope contains no elements.
     pub fn is_empty(&self) -> bool { self.num_bytes == 0 }
 
     pub fn check(&self) {
-        // #[cfg(test)]
-        {
-            assert!(self.head.height >= 1);
-            assert!(self.head.height < MAX_HEIGHT_U8 + 1);
+        assert!(self.head.height >= 1);
+        assert!(self.head.height < MAX_HEIGHT_U8 + 1);
 
-            let skip_over = &self.nexts[self.head.height as usize - 1];
-            // println!("Skip over skip chars {}, num bytes {}", skip_over.skip_chars, self.num_bytes);
-            assert!(skip_over.skip_chars <= self.num_bytes as usize);
-            assert!(skip_over.node.is_null());
+        let skip_over = &self.nexts[self.head.height as usize - 1];
+        // println!("Skip over skip chars {}, num bytes {}", skip_over.skip_chars, self.num_bytes);
+        assert!(skip_over.skip_chars <= self.num_bytes as usize);
+        assert!(skip_over.node.is_null());
 
-            // The offsets store the total distance travelled since the start.
-            let mut iter = [SkipEntry::new(); MAX_HEIGHT];
-            for i in 0..self.head.height {
-                // Bleh.
-                iter[i as usize].node = &self.head as *const Node as *mut Node;
-            }
-
-            let mut num_bytes: usize = 0;
-            let mut num_chars = 0;
-
-            for n in self.node_iter() {
-                // println!("visiting {:?}", n.as_str());
-                assert!(!n.str.is_empty() || std::ptr::eq(n, &self.head));
-                assert!(n.height <= MAX_HEIGHT_U8);
-                assert!(n.height >= 1);
-                n.str.check();
-
-                assert_eq!(count_chars(n.as_str_1()) + count_chars(n.as_str_2()), n.num_chars());
-                for (i, entry) in iter[0..n.height as usize].iter_mut().enumerate() {
-                    assert_eq!(entry.node as *const Node, n as *const Node);
-                    assert_eq!(entry.skip_chars, num_chars);
-
-                    // println!("replacing entry {:?} with {:?}", entry, n.nexts()[i].node);
-                    entry.node = n.nexts()[i].node;
-                    entry.skip_chars += n.nexts()[i].skip_chars;
-                }
-
-                num_bytes += n.str.len_bytes();
-                num_chars += n.num_chars();
-            }
-
-            for entry in iter[0..self.head.height as usize].iter() {
-                // println!("{:?}", entry);
-                assert!(entry.node.is_null());
-                assert_eq!(entry.skip_chars, num_chars);
-            }
-            
-            // println!("self bytes: {}, count bytes {}", self.num_bytes, num_bytes);
-            assert_eq!(self.num_bytes, num_bytes);
-            assert_eq!(self.len_chars(), num_chars);
+        // The offsets store the total distance travelled since the start.
+        let mut iter = [SkipEntry::new(); MAX_HEIGHT];
+        for i in 0..self.head.height {
+            // Bleh.
+            iter[i as usize].node = &self.head as *const Node as *mut Node;
         }
+
+        let mut num_bytes: usize = 0;
+        let mut num_chars = 0;
+
+        for n in self.node_iter() {
+            // println!("visiting {:?}", n.as_str());
+            assert!(!n.str.is_empty() || std::ptr::eq(n, &self.head));
+            assert!(n.height <= MAX_HEIGHT_U8);
+            assert!(n.height >= 1);
+            n.str.check();
+
+            assert_eq!(count_chars(n.as_str_1()) + count_chars(n.as_str_2()), n.num_chars());
+            for (i, entry) in iter[0..n.height as usize].iter_mut().enumerate() {
+                assert_eq!(entry.node as *const Node, n as *const Node);
+                assert_eq!(entry.skip_chars, num_chars);
+
+                // println!("replacing entry {:?} with {:?}", entry, n.nexts()[i].node);
+                entry.node = n.nexts()[i].node;
+                entry.skip_chars += n.nexts()[i].skip_chars;
+            }
+
+            num_bytes += n.str.len_bytes();
+            num_chars += n.num_chars();
+        }
+
+        for entry in iter[0..self.head.height as usize].iter() {
+            // println!("{:?}", entry);
+            assert!(entry.node.is_null());
+            assert_eq!(entry.skip_chars, num_chars);
+        }
+
+        // println!("self bytes: {}, count bytes {}", self.num_bytes, num_bytes);
+        assert_eq!(self.num_bytes, num_bytes);
+        assert_eq!(self.len_chars(), num_chars);
+    }
+
+    /// This method counts the number of bytes of memory allocated in the rope. This is purely for
+    /// debugging.
+    ///
+    /// Notes:
+    ///
+    /// - This method (its existence, its signature and its return value) is not considered part of
+    ///   the stable API provided by jumprope. This may disappear or change in point releases.
+    /// - This method walks the entire rope. It has time complexity O(n).
+    /// - If a rope is owned inside another structure, this method will double-count the bytes
+    ///   stored in the rope's head.
+    pub fn mem_size(&self) -> usize {
+        let mut nodes = self.node_iter();
+        let mut size = 0;
+        // The first node is the head. Count the actual head size.
+        size += std::mem::size_of::<Self>();
+        nodes.next(); // And discard it from the iterator.
+
+        for n in nodes {
+            let layout = Node::layout_with_height(n.height);
+            size += layout.size();
+        }
+
+        size
     }
 
     #[allow(unused)]
